@@ -2,7 +2,7 @@
 package BerkeleyDB;
 
 
-#     Copyright (c) 1997-2002 Paul Marquess. All rights reserved.
+#     Copyright (c) 1997-2003 Paul Marquess. All rights reserved.
 #     This program is free software; you can redistribute it and/or
 #     modify it under the same terms as Perl itself.
 #
@@ -17,7 +17,7 @@ use Carp;
 use vars qw($VERSION @ISA @EXPORT $AUTOLOAD
 		$use_XSLoader);
 
-$VERSION = '0.19';
+$VERSION = '0.21';
 
 require Exporter;
 #require DynaLoader;
@@ -85,6 +85,7 @@ BEGIN {
 	DB_ENCRYPT
 	DB_ENCRYPT_AES
 	DB_ENV_APPINIT
+	DB_ENV_AUTO_COMMIT
 	DB_ENV_CDB
 	DB_ENV_CDB_ALLDB
 	DB_ENV_CREATE
@@ -307,6 +308,12 @@ BEGIN {
 	DB_SWAPBYTES
 	DB_SYSTEM_MEM
 	DB_TEMPORARY
+	DB_TEST_ELECTINIT
+	DB_TEST_ELECTSEND
+	DB_TEST_ELECTVOTE1
+	DB_TEST_ELECTVOTE2
+	DB_TEST_ELECTWAIT1
+	DB_TEST_ELECTWAIT2
 	DB_TEST_POSTDESTROY
 	DB_TEST_POSTEXTDELETE
 	DB_TEST_POSTEXTOPEN
@@ -330,6 +337,7 @@ BEGIN {
 	DB_TXNVERSION
 	DB_TXN_ABORT
 	DB_TXN_APPLY
+	DB_TXN_BACKWARD_ALLOC
 	DB_TXN_BACKWARD_ROLL
 	DB_TXN_CKP
 	DB_TXN_FORWARD_ROLL
@@ -440,6 +448,30 @@ sub ParseParameters($@)
     return \%got ;
 }
 
+sub parseEncrypt
+{
+    my $got = shift ;
+
+
+    if (defined $got->{Encrypt}) {
+    	croak("Encrypt parameter must be a hash reference")
+            if !ref $got->{Encrypt} || ref $got->{Encrypt} ne 'HASH' ;
+
+	my %config = %{ $got->{Encrypt} } ;
+
+        my $p = BerkeleyDB::ParseParameters({
+					Password	=> undef,
+					Flags		=> undef,
+				}, %config);
+
+        croak("Must specify Password and Flags with Encrypt parameter")
+	    if ! (defined $p->{Password} && defined $p->{Flags});
+
+        $got->{"Enc_Passwd"} = $p->{Password};
+        $got->{"Enc_Flags"} = $p->{Flags};
+    }
+}
+
 use UNIVERSAL qw( isa ) ;
 
 sub env_remove
@@ -572,6 +604,8 @@ sub new
     #			[ -Cachesize	=> number ]
     #			[ -LockDetect	=>  ]
     #			[ -Verbose	=> boolean ]
+    #			[ -Encrypt	=> { Password => string, Flags => value}
+    #
     #			;
 
     my $pkg = shift ;
@@ -587,6 +621,7 @@ sub new
 					LockDetect     	=> 0,
 					Verbose		=> 0,
 					Config		=> undef,
+					Encrypt		=> undef,
 					}, @_) ;
 
     if (defined $got->{ErrFile}) {
@@ -609,7 +644,7 @@ sub new
         @BerkeleyDB::a = () ;
 	my $k = "" ; my $v = "" ;
 	while (($k, $v) = each %config) {
-	    if ($BerkeleyDB::db_version >= 3.1 && ! $valid_config_keys{$k} ) {
+	    if ($BerkeleyDB::db_version >= 3.1 && ! $valid_config_keys{$k} ){
 	        $BerkeleyDB::Error = "illegal name-value pair: $k $v\n" ; 
                 croak $BerkeleyDB::Error ;
 	    }
@@ -620,6 +655,8 @@ sub new
 	    if @BerkeleyDB::a ;
     }
 
+    BerkeleyDB::parseEncrypt($got);
+
     my ($addr) = _db_appinit($pkg, $got) ;
     my $obj ;
     $obj = bless [$addr] , $pkg if $addr ;
@@ -691,6 +728,7 @@ sub new
 			Env		=> undef,
 			#Tie 		=> undef,
 			Txn		=> undef,
+			Encrypt		=> undef,
 
 			# Hash specific
 			Ffactor		=> 0,
@@ -714,6 +752,8 @@ sub new
     croak("-Tie needs a reference to a hash")
 	if defined $got->{Tie} and $got->{Tie} !~ /HASH/ ;
 
+    BerkeleyDB::parseEncrypt($got);
+
     my ($addr) = _db_open_hash($self, $got);
     my $obj ;
     if ($addr) {
@@ -753,6 +793,7 @@ sub new
 			Env		=> undef,
 			#Tie 		=> undef,
 			Txn		=> undef,
+			Encrypt		=> undef,
 
 			# Btree specific
 			Minkey		=> 0,
@@ -770,6 +811,8 @@ sub new
     croak("-Tie needs a reference to a hash")
 	if defined $got->{Tie} and $got->{Tie} !~ /HASH/ ;
 
+    BerkeleyDB::parseEncrypt($got);
+
     my ($addr) = _db_open_btree($self, $got);
     my $obj ;
     if ($addr) {
@@ -809,6 +852,7 @@ sub new
 			Env		=> undef,
 			#Tie 		=> undef,
 			Txn		=> undef,
+			Encrypt		=> undef,
 
 			# Recno specific
 			Delim		=> undef,
@@ -831,6 +875,8 @@ sub new
 	if $got->{ArrayBase} != 1 and $got->{ArrayBase} != 0 ;
 
 
+    BerkeleyDB::parseEncrypt($got);
+
     $got->{Fname} = $got->{Filename} if defined $got->{Filename} ;
 
     my ($addr) = _db_open_recno($self, $got);
@@ -872,6 +918,7 @@ sub new
 			Env		=> undef,
 			#Tie 		=> undef,
 			Txn		=> undef,
+			Encrypt		=> undef,
 
 			# Queue specific
 			Len		=> undef,
@@ -892,6 +939,8 @@ sub new
     croak("ArrayBase can only be 0 or 1, parsed $got->{ArrayBase}")
 	if $got->{ArrayBase} != 1 and $got->{ArrayBase} != 0 ;
 
+    BerkeleyDB::parseEncrypt($got);
+
     $got->{Fname} = $got->{Filename} if defined $got->{Filename} ;
 
     my ($addr) = _db_open_queue($self, $got);
@@ -989,6 +1038,7 @@ sub new
 			Env		=> undef,
 			#Tie 		=> undef,
 			Txn		=> undef,
+			Encrypt		=> undef,
 
 		      }, @_) ;
 
@@ -1001,6 +1051,8 @@ sub new
     croak("-Tie needs a reference to a hash")
 	if defined $got->{Tie} and $got->{Tie} !~ /HASH/ ;
 
+    BerkeleyDB::parseEncrypt($got);
+
     my ($addr, $type) = _db_open_unknown($got);
     my $obj ;
     if ($addr) {
@@ -1096,11 +1148,9 @@ sub CLEAR
 {
     my $self = shift ;
     my ($key, $value) = (0, 0) ;
-    my $cursor = $self->db_cursor() ;
+    my $cursor = $self->_db_write_cursor() ;
     while ($cursor->c_get($key, $value, BerkeleyDB::DB_PREV()) == 0) 
 	{ $cursor->c_del() }
-    #1 while $cursor->c_del() == 0 ;
-    # cursor will self-destruct
 }
 
 #sub DESTROY
@@ -1376,17 +1426,28 @@ sub db_cursor
     return $obj ;
 }
 
-sub db_join
+sub _db_write_cursor
 {
-    croak 'Usage: $db->BerkeleyDB::Common::db_join([cursors], flags=0)'
-	if @_ < 2 || @_ > 3 ;
     my $db = shift ;
-    my ($addr) = $db->_db_join(@_) ;
+    my ($addr) = $db->__db_write_cursor(@_) ;
     my $obj ;
     $obj = bless [$addr, $db] , "BerkeleyDB::Cursor" if $addr ;
     return $obj ;
 }
 
+sub db_join
+{
+    croak 'Usage: $db->BerkeleyDB::db_join([cursors], flags=0)'
+	if @_ < 2 || @_ > 3 ;
+    my $db = shift ;
+    croak 'db_join: first parameter is not an array reference'
+	if ! ref $_[0] || ref $_[0] ne 'ARRAY';
+    my ($addr) = $db->_db_join(@_) ;
+    my $obj ;
+    $obj = bless [$addr, $db, $_[0]] , "BerkeleyDB::Cursor" if $addr ;
+    return $obj ;
+}
+
 package BerkeleyDB::Cursor ;
 
 sub c_close
